/***************************************************************************
                          simulator.cpp  -  description
                             -------------------
    begin                : Mon Jun 2 2003
    copyright            : (C) 2003 by Marcus Beranek
    email                : marcus@beranek.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "simulator.h"
#include "blockbtn.h"
#include "blocklist.h"
#include "successor.h"
#include "successorlist.h"


#include <qvariant.h>
#include <qscrollview.h>
#include <qframe.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qsplitter.h>
#include <qtextedit.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qpixmap.h>
#include <qrect.h>

Simulator::Simulator(QWidget *parent, const char *name ) : ViewPlugin(parent,name)
{
	if ( !name )
		setName( "Simulator" );
	setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)2, (QSizePolicy::SizeType)2, 0, 0, sizePolicy().hasHeightForWidth() ) );
	Form1Layout = new QHBoxLayout( this, 11, 6, "Form1Layout");

	splitter4 = new QSplitter( this, "splitter4" );
	splitter4->setMinimumSize( QSize( 612, 600 ) );
	splitter4->setOrientation( QSplitter::Horizontal );

	frame6 = new QFrame( splitter4, "frame6" );
	frame6->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)7, 0, 0, frame6->sizePolicy().hasHeightForWidth() ) );
	frame6->setMinimumSize( QSize( 50, 600 ) );
	frame6->setFrameShape( QFrame::StyledPanel );
	frame6->setFrameShadow( QFrame::Raised );
	frame6Layout = new QVBoxLayout( frame6, 11, 6, "frame6Layout");

	textLabel1 = new QLabel( frame6, "textLabel1" );
	textLabel1->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)5, 0, 0, textLabel1->sizePolicy().hasHeightForWidth() ) );
	textLabel1->setMinimumSize( QSize( 0, 20 ) );
	textLabel1->setMaximumSize( QSize( 32767, 40 ) );
	frame6Layout->addWidget( textLabel1 );

	scrollView2 = new QScrollView( frame6, "scrollView2" );
	scrollView2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)7, 0, 0, scrollView2->sizePolicy().hasHeightForWidth() ) );
	scrollView2->setVScrollBarMode( QScrollView::AlwaysOn );
	scrollView2->viewport()->setPaletteBackgroundColor( parent->paletteBackgroundColor() );
	
	frame6Layout->addWidget( scrollView2 );

	frame7 = new QFrame( splitter4, "frame7" );
	frame7->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)7, 0, 0, frame7->sizePolicy().hasHeightForWidth() ) );
	frame7->setMinimumSize( QSize( 500, 600 ) );
	frame7->setFrameShape( QFrame::StyledPanel );
	frame7->setFrameShadow( QFrame::Raised );
	frame7Layout = new QHBoxLayout( frame7, 11, 6, "frame7Layout");

	textEdit1 = new QTextEdit( frame7, "textEdit1" );
	textEdit1->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)7, 0, 0, textEdit1->sizePolicy().hasHeightForWidth() ) );
	textEdit1->setPaletteForegroundColor( QColor( 0, 255, 0 ) );
	textEdit1->setPaletteBackgroundColor( QColor( 0, 0, 0 ) );
	QFont textEdit1_font(  textEdit1->font() );
	textEdit1_font.setFamily( "Courier" );
	textEdit1_font.setPointSize( 12 );
	textEdit1_font.setBold( TRUE );
	textEdit1->setFont( textEdit1_font );
	textEdit1->setCursor( QCursor( 4 ) );
	textEdit1->setTextFormat( QTextEdit::PlainText );
	textEdit1->setReadOnly( TRUE );
	frame7Layout->addWidget( textEdit1 );

	frame6_2 = new QFrame( splitter4, "frame6_2" );
	frame6_2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)7, 0, 0, frame6_2->sizePolicy().hasHeightForWidth() ) );
	frame6_2->setMinimumSize( QSize( 50, 600 ) );
	frame6_2->setFrameShape( QFrame::StyledPanel );
	frame6_2->setFrameShadow( QFrame::Raised );
	frame6_2Layout = new QVBoxLayout( frame6_2, 11, 6, "frame6_2Layout");

	textLabel1_2 = new QLabel( frame6_2, "textLabel1_2" );
	textLabel1_2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)5, 0, 0, textLabel1_2->sizePolicy().hasHeightForWidth() ) );
	textLabel1_2->setMinimumSize( QSize( 0, 20 ) );
	textLabel1_2->setMaximumSize( QSize( 32767, 40 ) );
	frame6_2Layout->addWidget( textLabel1_2 );

	scrollView2_2 = new QScrollView( frame6_2, "scrollView2_2" );
	scrollView2_2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)7, 0, 0, scrollView2_2->sizePolicy().hasHeightForWidth() ) );
	scrollView2_2->setVScrollBarMode( QScrollView::AlwaysOn );
	scrollView2_2->viewport()->setPaletteBackgroundColor( parent->paletteBackgroundColor() );
	
	buttonbox_send = new QFrame( scrollView2_2->viewport() );
	buttonbox_receive = new QFrame( scrollView2->viewport() );
	scrollView2_2->addChild(buttonbox_send);
	scrollView2->addChild(buttonbox_receive);	
	send_Layout = new QVBoxLayout( buttonbox_send, 11, 6, "scrollView2_2Layout");
	receive_Layout = new QVBoxLayout( buttonbox_receive, 11, 6, "scrollView2Layout");

	max_width_send=0;
	max_width_receive=0;
	
	frame6_2Layout->addWidget( scrollView2_2 );
	Form1Layout->addWidget( splitter4 );
	languageChange();
	resize( QSize(789, 623).expandedTo(minimumSizeHint()) );
	
	
}

Simulator::~Simulator()
{
}


void Simulator::init( ShowState show)
{
  //getButtons(0);
  blockSelected(0);
}


void Simulator::clear()
{

	bl->clear();

	// we are radical... 
	delete buttonbox_send;
	delete buttonbox_receive;
	
	// ... and constructive ... ;)
	buttonbox_send = new QFrame( scrollView2_2->viewport() );
	buttonbox_receive = new QFrame( scrollView2->viewport() );
	scrollView2_2->addChild(buttonbox_send);
	scrollView2->addChild(buttonbox_receive);	
	send_Layout = new QVBoxLayout( buttonbox_send, 11, 6, "scrollView2_2Layout");
	receive_Layout = new QVBoxLayout( buttonbox_receive, 11, 6, "scrollView2Layout");
	
//	if( scrollView2_2->width() > max_width_send ){
//		send_Layout->setGeometry( QRect::QRect(0,0,scrollView2_2->width(), buttonbox_send->height() ) );
//		max_width_send = scrollView2_2->width();
//	
//	}

	buttonbox_send->show();
	buttonbox_receive->show();
	
  
}


void Simulator::getButtons( int blockid )
{
	int z = bl->find( new Block( blockid ) );
	
	if( z > -1 ){
		
		Block* b = (Block*) bl->at( z );
		
		if( blockid == 0 ){
			textEdit1->setText( b->Command + ":\n\n" );
		} else{
			textEdit1->setText(textEdit1->text() + b->Example + "\n" );
		}
		textEdit1->setCursorPosition( textEdit1->paragraphs() -1 , textEdit1->paragraphLength( textEdit1->paragraphs() - 1 ) );
		//textEdit1->setCursorPosition( 2, 2);
		
		SuccessorList sl = b->getFollowedby();
		if( ! sl.isEmpty() ){
			// read all followers and create buttons for them:
			Successor* s;
			for( s = sl.first(); s; s = sl.next() ){
				std::cout << "Block " << b->Command << " folowed by " << s->ID << endl;
				int y = bl->find( new Block( s->ID ) );
				if( y > -1 ){
					Block* fb = (Block*) bl->at(y);
					QWidget* p;
					QVBoxLayout* pl;
					if (fb->Type == Block::Send ){
						p = buttonbox_send;
						pl = send_Layout;
					} else {
						p = buttonbox_receive;
						pl = receive_Layout;
					}
					BlockBtn* bb = new BlockBtn( p);
					pl->addWidget( bb );
					bb->setID( s->ID );
					bb->setText( fb->Command );
					bb->show();
					bbl.append( bb );
					
					connect( bb, SIGNAL( blockSelected( int ) ), this, SLOT( blockSelected(int) ) );
				}
			}
			QSpacerItem* spacer_2 = new QSpacerItem( 20, 440, QSizePolicy::Minimum, QSizePolicy::Expanding );
			send_Layout->addItem( spacer_2 );
			
			QSpacerItem* spacer = new QSpacerItem( 20, 440, QSizePolicy::Minimum, QSizePolicy::Expanding );
			receive_Layout->addItem( spacer );
			
			spl.append (spacer_2 );
			spl.append (spacer );

			
		}
		
	}
}


void Simulator::blockSelected( int blockid )
{
	// we are radical... 
	delete buttonbox_send;
	delete buttonbox_receive;
	
	// ... and constructive ... ;)
	buttonbox_send = new QFrame( scrollView2_2->viewport() );
	buttonbox_receive = new QFrame( scrollView2->viewport() );
	scrollView2_2->addChild(buttonbox_send);
	scrollView2->addChild(buttonbox_receive);	
	send_Layout = new QVBoxLayout( buttonbox_send, 11, 6, "scrollView2_2Layout");
	receive_Layout = new QVBoxLayout( buttonbox_receive, 11, 6, "scrollView2Layout");
	
//	if( scrollView2_2->width() > max_width_send ){
//		send_Layout->setGeometry( QRect::QRect(0,0,scrollView2_2->width(), buttonbox_send->height() ) );
//		max_width_send = scrollView2_2->width();
//	
//	}

	buttonbox_send->show();
	buttonbox_receive->show();
	
	getButtons( blockid );
	
}


/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void Simulator::languageChange()
{
	setCaption( tr( "Form1" ) );
	textLabel1->setText( tr( "Receive" ) );
	//pushButton1->setText( tr( "pushButton1" ) );
	//pushButton2->setText( tr( "pushButton2" ) );
	//textEdit1->setText( tr( "HELO" ) );
	textLabel1_2->setText( tr( "Send" ) );
	//pushButton1_2->setText( tr( "pushButton1" ) );
	//pushButton2_2->setText( tr( "pushButton2" ) );
}

extern "C" ViewPlugin* create(QWidget *parent, const char *name){
	return new Simulator( parent, name );
}

extern "C" void destroy( ViewPlugin* p){
	delete p;
}

