/***************************************************************************
                          treedisplay.cpp  -  description
                             -------------------
    begin                : Sat May 24 2003
    copyright            : (C) 2003 by Marcus Beranek
    email                : marcus@beranek.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "blockdisplay.h"
#include "blockline.h"
#include "blockdisplaylist.h"
#include "blocklinelist.h"
#include "blocklist.h"

#include "display.h"

#include "treedisplay.h"

#include "viewplugin.h"

#include <qptrlist.h>
#include <qpainter.h>
#include <qscrollview.h>
#include <qlayout.h>
#include <qrect.h>

//TreeDisplay::TreeDisplay(QWidget *parent, const char *name ) : QWidget(parent,name)
TreeDisplay::TreeDisplay(QWidget *parent, const char *name ) : ViewPlugin(parent,name)
{

	b = new BlockDisplayList();
	l = new BlockLineList();
	//bl = new BlockList();

	b->setAutoDelete(true);
	l->setAutoDelete(true);

	//screen = new QWidget( this );
	tree = new QScrollView( this, "treebox" );
	
	
	treeLayout = new QHBoxLayout( this, 11, 6, "treeLayout");
	
	tree->setMinimumSize( QSize( 300, 300 ) );
	tree->setFrameShape( QFrame::StyledPanel );
	tree->setFrameShadow( QFrame::Raised );
	tree->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
	tree->setGeometry( QRect( 0, 0, 800, 600 ) );

	tree->setHScrollBarMode( QScrollView::AlwaysOn );
	tree->setVScrollBarMode( QScrollView::AlwaysOn );

	treeLayout->addWidget(tree);
	//screen = new QWidget( tree->viewport() );
	//tree->addChild( screen );
	
	//t_disp = new Display( tree->viewport() );
	t_disp = new TDisplay( tree->viewport() );
	tree->addChild( t_disp );	
	
	t_disp->setBlockDisplayList( b );
	t_disp->setBlockLineList( l );
	
	w = 800;
	h = 600;

	max_button_width = 50;
	max_button_height = 20;

	spacex = 50;
	spacey = 10;

	col = 0;
	row = 0;


}
//---------------------------------------------------------------------------



TreeDisplay::~TreeDisplay()
{

}
//---------------------------------------------------------------------------



void TreeDisplay::addBlockList( BlockList* b )
{
	//bl = b;
}
//---------------------------------------------------------------------------

void TreeDisplay::setBlockList( BlockList* b )
{
	bl = b;
}
//---------------------------------------------------------------------------



void TreeDisplay::addBlockCoord(QString n, int id, int blockid, int x, int y)
{
	b->append( new BlockDisplay(n,id,blockid,x,y) );

	w = ( (x + 200) > w ) ? ( x + 200 ) : (w);
	h = ( (y + 200) > h ) ? ( y + 200 ) : (h);

	//screen->resize( w, h );
	//screen->update();
	t_disp->resize( w, h );
	t_disp->update();

}
//---------------------------------------------------------------------------



void TreeDisplay::addLine(int id1, int id2)
{
	if( id1 != id2 ){
	  
		// ursprnglich:
		// l->append( new BlockLine(id1, id2) );
		// recalculateLines();
		//
		// aber das hier ist schneller, da nich bei jeder neuen Linie _alle_ Linien
		// neu berechnet werden mssen, sonder nur fr die neue...
		//
		BlockLine* nl = new BlockLine(id1, id2);

		int pos1 = b->find( new BlockDisplay( id1 ) );
		int pos2 = b->find( new BlockDisplay( id2 ) );

		if ( ( pos1 > -1 ) && ( pos2 > -1 ) ) {
			nl->setLineBetween( (BlockDisplay*) b->at(pos1), (BlockDisplay*) b->at(pos2) );
			l->append( nl );
		}

	}
}
//---------------------------------------------------------------------------



void TreeDisplay::recalculateLines()
{
	BlockLine* line;

	for( line = l->first(); line; line = l->next()){
		int i1 = line->id1;
		int i2 = line->id2;

		// find these ids in b:
		int pos1 = b->find( new BlockDisplay( i1 ) );
		int pos2 = b->find( new BlockDisplay( i2 ) );

		if ( ( pos1 > -1 ) && ( pos2 > -1 ) ) {
			// beide existieren...
			line->setLineBetween( (BlockDisplay*) b->at(pos1), (BlockDisplay*) b->at(pos2) );
		}

	}

}
//---------------------------------------------------------------------------



void TreeDisplay::clear()
{
	b->clear();
	l->clear();
	bl->clear();

	max_button_width = 50;
	max_button_height = 20;

	spacex = 50;
	spacey = 10;

	col = 0;
	row = 0;

	w = 800;
	h = 600;

	//screen->resize( w, h );
	t_disp->resize( w, h );

}
//---------------------------------------------------------------------------



// void TreeDisplay::paintEvent( QPaintEvent * )
// {
	// //QPainter p( this );
	// QPainter p( screen );
	// p.setBrush( white );
	// p.drawRect( 0,0, w, h );
// 
// 
	// p.setPen( black );
// 
	// // 1. alle Blocks malen:
	// BlockDisplay* disp;
	// for( disp = b->first(); disp; disp = b->next()){
		// // TODO: rectangle around text.
		// //
		// // unfortnatly, the coordinates are wrong:
		// // p.drawRect( disp->left_x, disp->left_y, disp->right_x, disp->right_y );
		// //
		// p.drawText( disp->x1 + 10, disp->y1, disp->Name );
	// }
// 
	// // 2. alle Linien ziehen:
	// BlockLine* line;
	// for( line = l->first(); line; line = l->next()){
		// p.setPen( NoPen );
		// p.moveTo( line->x1, line->y1 );
		// p.setPen( black );
		// p.lineTo( line->x2, line->y2 );
	// }
	// 
	// p.setPen( NoPen );
	// 
// }
//---------------------------------------------------------------------------



QSizePolicy TreeDisplay::sizePolicy() const
{
	return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
}
//---------------------------------------------------------------------------


void TreeDisplay::addBlock(Block * block)
{
	// checkBlock( "addBlock", block );
	Block* x = new Block( *block);
	
	int w_tmp = x->Name.length() * 12 + 20 ;
	max_button_width = ( w_tmp > max_button_width ) ? ( w_tmp ) : ( max_button_width );
	max_button_height = 20;
	
	x->fby.clear();      // Problems in Block-CC with QPtrList...
	x->fls.clear();      // Problems in Block-CC with QPtrList...
	x->typical.clear();  // Problems in Block-CC with QPtrList...
	
	bl->append( x );
}
//---------------------------------------------------------------------------



/** No descriptions */
void TreeDisplay::init( ViewPlugin::ShowState show )
{
	flowChart( bl, bl->first(), show, 0, 0 );
	update();
}
//---------------------------------------------------------------------------


void TreeDisplay::flowChart( BlockList* blist, Block* b, ViewPlugin::ShowState show, int x_pos, int y_pos, int oldID )
{
	//
	// we are parsing the tree via depth-search.
	//
	// basically, we do the following:
	//
	//    o display block, line to previous block
	//    o make the newID the oldID,
	//    o fetch all children of block, if we didn't have allready visited this block...
	//

	Block* block;

	// TODO: Check coordinates...

	// Search for the block b ...
	int z = blist->find( b );
	if ( z > -1 ){
		block = blist->at( z );
		SuccessorList sl;
		Successor* f;

		// draw button and connecting-line...
		treeview_x_coord = col * (max_button_width + spacex);
		treeview_y_coord = row * (max_button_height + spacey);
		//std::cout << block->Command << " col: " << col << " x: " << treeview_x_coord << " row: " << row << " y: " << treeview_y_coord <<  //std::endl;


		addBlockCoord( block->Command, newID, block->ID, treeview_x_coord, treeview_y_coord+30 );
		addLine( oldID, newID );


		oldID=newID;
		newID++;


		// remember maximum
		int xx = (col+1) * (max_button_width + spacex);
		int yy = (row+1) * (max_button_height + spacey);
		max_x = ( xx > max_x ) ? ( xx ) : ( max_x );
		max_y = ( yy > max_y ) ? ( yy ) : ( max_y );

		// Lineto-position:
		int lineto_x = treeview_x_coord;
		int lineto_y = treeview_y_coord + ((int) (max_button_height / 2 ));


		// block-end-position:
		int new_x_pos = treeview_x_coord + max_button_width;
		int new_y_pos = treeview_y_coord + ((int) (max_button_height / 2 ));

		// increase y:
		row++;

		// full protocol or typical only...
		switch( show ){
		case ViewPlugin::ShowAll: {
				sl = block->getFollowedby();
				break;
			}
		case ViewPlugin::ShowTypical: {
				sl = block->getTypicalFollowedby();
				break;
			}
		}


		if ( block->visited ){
			// nothin...

		} else { // block not visited
			block->visited = true;
			// if we have children
			if( ! sl.isEmpty() ){

				// increase x:
				col++;

				// decrease y (cause we have already increased it after button-creation):
				row--;

				// parse every child:
				for( Successor* sux = sl.first(); sux; sux = sl.next() ){
					//std::cout << "PLUGIN treedisplay: " << block->Command << " followed by " << sux->ID << endl;
					int r = blist->find( new Block( sux->ID ) );
					if ( r > -1 ){
						Block* v = blist->at( r );
						flowChart( blist, v, show, new_x_pos, new_y_pos, oldID );
					} // end if r > -1

				} // end for sl

				// decrease x, cause we're now stepping back to the left again
				col--;

			}

		} // end if block not visited

	} // end if z > -1

}
//---------------------------------------------------------------------------



extern "C" ViewPlugin* create(QWidget *parent, const char *name){
	return new TreeDisplay( parent, name );
}

extern "C" void destroy( ViewPlugin* p){
	delete p;
}

